"""
Module realize accounts mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload

from .base import Backport3MixinStub


class AccountMixin(Backport3MixinStub):
    """Account Mixin to work with accounts & tokens."""

    def createAccount(
        self, email: str, organizationName: str, password: str, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create account.

        Args:
            email: email
            organizationName: organization name
            password: password

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created token.

        """
        payload = RequestPayload.buildJson(
            body={"email": email, "organization_name": organizationName, "password": password}
        )
        return self.makeRequest(url=f"{self.baseUri}/accounts", method="POST", body=payload, **kwargs)

    def getAccountData(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get account email, organization name and status.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with account data.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/account", method="GET", **kwargs)

    def getTokens(
        self,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets account tokens. Every token is represented by *id* and *token data*.

        Args:
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with tokens.

        """
        queries = {"page": page, "page_size": pageSize}
        return self.makeAuthRequest(url=f"{self.baseUri}/account/tokens", method="GET", queryParams=queries, **kwargs)

    def createToken(self, tokenData: Optional[str] = "", **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates a token with token data. New token's *id* is returned.

        Args:
            tokenData: token data for token

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with token id.

        """
        payload = RequestPayload.buildJson(body={"token_data": tokenData})
        return self.makeAuthRequest(url=f"{self.baseUri}/account/tokens", method="POST", body=payload, **kwargs)

    def deleteTokens(self, tokens: List[str], **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function remove a tokens' list.

        Args:
            tokens: list of token ids.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(body={"tokens": tokens})
        return self.makeAuthRequest(url=f"{self.baseUri}/account/tokens", method="DELETE", body=payload, **kwargs)

    def getToken(self, tokenId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets *token data* of a token, which corresponds to *tokenId*.

        Args:
            tokenId: token *id*

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with token data.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/account/tokens/{tokenId}", method="GET", **kwargs)

    def patchTokenData(self, tokenId: str, tokenData: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function patches a token data for the token, which corresponds to *tokenId*.

        Args:
            tokenId: token *id*
            tokenData: token data

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(body={"token_data": tokenData})
        return self.makeAuthRequest(
            url=f"{self.baseUri}/account/tokens/{tokenId}", method="PATCH", body=payload, **kwargs
        )

    def deleteToken(self, tokenId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function remove a token, which corresponds to *tokenId*.

        Args:
            tokenId: token *id*

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/account/tokens/{tokenId}", method="DELETE", **kwargs)
