"""
Module realize handlers mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.python_matcher.match_objects import BinaryReference

from .base import Backport3MixinStub


class HandlerMixin(Backport3MixinStub):
    """Handler Mixin to work with handlers."""

    def extractPerson(
        self,
        body: bytes,
        userData: Optional[str] = None,
        externalId: Optional[str] = None,
        warpedImage: bool = False,
        noCache: bool = False,
        contentType: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract a person from an image. This request is just a simple way to tell "extract a descriptor then create
        person with this descriptor, attach person to a list".

        Args:
            body: image bytes
            userData: user data for creating a person
            externalId: external id for creating a person
            warpedImage: Determines, whether an input image is a warped or an arbitrary one. Exact image warping
                algorithm is proprietary and this flag is intended for VisionLabs front-end tools only.

                The warped image has the following properties:

                * size is always 250x250 pixels;

                * color format is always RGB;

                * single face in a photo;

                * the face is always centered and rotated so that imaginary line between the eyes is horizontal.
            noCache: A boolean, whether to use cache or no
            contentType: image mime type, if contentType is not set, will try to determine it by ourselves
                raise ValueError if mimetype of content is not matches with acceptable formats
                (Available mimetypes are: image/jpeg, image/png, image/gif, image/bmp, image/tiff,
                application/x-vl-xpk, application/x-vl-face-descriptor, image/x-portable-pixmap)
            login: email (login)
            password: password
            token: token
            lunaRequestId: Luna-Request-Id
            asyncRequest: mode for request async or blocking
            raiseError: if request fails, LunaApiException is raised
            totalTimeout: request's processing timeout in seconds.
            connectTimeout: connection timeout in seconds.
            sockConnectTimeout: socket connection timeout in seconds.
            sockReadTimeout: socket chunk read timeout in seconds.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        if body is None:
            raise ValueError

        if contentType is None:
            contentType = self.getContentType(body)
        headers = {"Content-Type": contentType}

        queries = {
            "warped_image": int(warpedImage),
            "user_data": userData,
            "external_id": externalId,
            "no_cache": int(noCache),
        }

        return self.makeAuthRequest(
            url=f"{self.baseUri}/handlers/extractor",
            method="POST",
            queryParams=queries,
            body=body,
            headers=headers,
            **kwargs,
        )

    def getExtractorHandler(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get extractor handler info.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/extractor", method="GET", **kwargs)

    def updateExtractorHandler(
        self,
        listId: Optional[str] = None,
        estimateAttributes: Optional[int] = None,
        estimateEmotions: Optional[int] = None,
        estimateEthnicities: Optional[int] = None,
        estimateQuality: Optional[int] = None,
        estimateHeadPose: Optional[int] = None,
        extractExif: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        pitchThreshold: Optional[int] = None,
        scoreThreshold: Optional[float] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch extractor handler with specified parameters.

        Args:
            listId: list id to link extracted faces to
            estimateAttributes: whether to estimate basic attributes
            estimateEmotions: whether to estimate emotions
            estimateEthnicities: whether to estimate ethnicitites
            estimateQuality: whether to estimate quality
            estimateHeadPose: whether to estimate head pose
            extractExif: whether to extract exif
            yawThreshold: maximum deviation yaw angle
            rollThreshold: maximum deviation roll angle
            pitchThreshold: maximum deviation pitch angle
            scoreThreshold: score threshold

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        data = {}
        if listId is not None:
            data["list_id"] = listId
        if estimateAttributes is not None:
            data["estimate_attributes"] = estimateAttributes
        if estimateEmotions is not None:
            data["estimate_emotions"] = estimateEmotions
        if estimateEthnicities is not None:
            data["estimate_ethnicities"] = estimateEthnicities
        if estimateQuality is not None:
            data["estimate_quality"] = estimateQuality
        if estimateHeadPose is not None:
            data["estimate_head_pose"] = estimateHeadPose
        if extractExif is not None:
            data["extract_exif"] = extractExif
        if yawThreshold is not None:
            data["yaw_threshold"] = yawThreshold
        if rollThreshold is not None:
            data["roll_threshold"] = rollThreshold
        if pitchThreshold is not None:
            data["pitch_threshold"] = pitchThreshold
        if scoreThreshold is not None:
            data["score_threshold"] = scoreThreshold

        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/extractor", method="PATCH", body=payload, **kwargs)

    def identifyPerson(
        self, body: bytes, warpedImage: bool = False, noCache: bool = False, contentType: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract a person from an image. This request is just a simple way to tell "identify a descriptor then create
        person with this descriptor, attach person to a list".

        Args:
            body: image bytes
            warpedImage: Determines, whether an input image is a warped or an arbitrary one. Exact image warping
                algorithm is proprietary and this flag is intended for VisionLabs front-end tools only.

                The warped image has the following properties:

                * size is always 250x250 pixels;

                * color format is always RGB;

                * single face in a photo;

                * the face is always centered and rotated so that imaginary line between the eyes is horizontal.
            noCache: A boolean, whether to use cache or no
            contentType: image mime type, if contentType is not set, will try to determine it by ourselves
                raise ValueError if mimetype of content is not matches with acceptable formats
                (Available mimetypes are: image/jpeg, image/png, image/gif, image/bmp, image/tiff,
                application/x-vl-xpk, application/x-vl-face-descriptor, image/x-portable-pixmap)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        if body is None:
            raise ValueError

        if contentType is None:
            contentType = self.getContentType(body)
        headers = {"Content-Type": contentType}

        queries = {
            "warped_image": int(warpedImage),
            "no_cache": int(noCache),
        }

        return self.makeAuthRequest(
            url=f"{self.baseUri}/handlers/identify",
            method="POST",
            queryParams=queries,
            body=body,
            headers=headers,
            **kwargs,
        )

    def getIdentifyHandler(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get identify handler info.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/identify", method="GET", **kwargs)

    def updateIdentifyHandler(  # pylint: disable-msg=R0912
        self,
        listId: Optional[str] = None,
        limit: Optional[int] = None,
        estimateAttributes: Optional[int] = None,
        estimateEmotions: Optional[int] = None,
        estimateEthnicities: Optional[int] = None,
        estimateQuality: Optional[int] = None,
        estimateHeadPose: Optional[int] = None,
        extractExif: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        pitchThreshold: Optional[int] = None,
        scoreThreshold: Optional[float] = None,
        identifyThreshold: Optional[float] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch identify handler with specified parameters.

        Args:
            listId: matching limit
            limit: list id to link extracted faces to
            estimateAttributes: whether to estimate basic attributes
            estimateEmotions: whether to estimate emotions
            estimateEthnicities: whether to estimate ethnicitites
            estimateQuality: whether to estimate quality
            estimateHeadPose: whether to estimate head pose
            extractExif: whether to extract exif
            yawThreshold: maximum deviation yaw angle
            rollThreshold: maximum deviation roll angle
            pitchThreshold: maximum deviation pitch angle
            scoreThreshold: score threshold
            identifyThreshold: identify threshold

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        data = {}
        if listId is not None:
            data["list_id"] = listId
        if limit is not None:
            data["limit"] = limit
        if estimateAttributes is not None:
            data["estimate_attributes"] = estimateAttributes
        if estimateEmotions is not None:
            data["estimate_emotions"] = estimateEmotions
        if estimateEthnicities is not None:
            data["estimate_ethnicities"] = estimateEthnicities
        if estimateQuality is not None:
            data["estimate_quality"] = estimateQuality
        if estimateHeadPose is not None:
            data["estimate_head_pose"] = estimateHeadPose
        if extractExif is not None:
            data["extract_exif"] = extractExif
        if yawThreshold is not None:
            data["yaw_threshold"] = yawThreshold
        if rollThreshold is not None:
            data["roll_threshold"] = rollThreshold
        if pitchThreshold is not None:
            data["pitch_threshold"] = pitchThreshold
        if scoreThreshold is not None:
            data["score_threshold"] = scoreThreshold
        if identifyThreshold is not None:
            data["identify_threshold"] = identifyThreshold

        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/identify", method="PATCH", body=payload, **kwargs)

    def verifyPerson(
        self,
        body: bytes,
        personId: str,
        warpedImage: bool = False,
        noCache: bool = False,
        contentType: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract a person from an image. This request is just a simple way to tell "verify a descriptor then create
        person with this descriptor, attach person to a list".

        Args:
            body: image bytes
            personId: person id
            noCache: A boolean, whether to use cache or no
            warpedImage: Determines, whether an input image is a warped or an arbitrary one. Exact image warping
                algorithm is proprietary and this flag is intended for VisionLabs front-end tools only.

                The warped image has the following properties:

                * size is always 250x250 pixels;

                * color format is always RGB;

                * single face in a photo;

                * the face is always centered and rotated so that imaginary line between the eyes is horizontal.
            contentType: image mime type, if contentType is not set, will try to determine it by ourselves
                raise ValueError if mimetype of content is not matches with acceptable formats
                (Available mimetypes are: image/jpeg, image/png, image/gif, image/bmp, image/tiff,
                application/x-vl-xpk, application/x-vl-face-descriptor, image/x-portable-pixmap)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        if body is None:
            raise ValueError

        if contentType is None:
            contentType = self.getContentType(body)
        headers = {"Content-Type": contentType}

        queries = {
            "warped_image": int(warpedImage),
            "person_id": personId,
            "no_cache": int(noCache),
        }

        return self.makeAuthRequest(
            url=f"{self.baseUri}/handlers/verify",
            method="POST",
            queryParams=queries,
            body=body,
            headers=headers,
            **kwargs,
        )

    def getVerifyHandler(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get verify handler info.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/verify", method="GET", **kwargs)

    def updateVerifyHandler(
        self,
        estimateAttributes: Optional[int] = None,
        estimateEmotions: Optional[int] = None,
        estimateEthnicities: Optional[int] = None,
        estimateQuality: Optional[int] = None,
        estimateHeadPose: Optional[int] = None,
        extractExif: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        pitchThreshold: Optional[int] = None,
        scoreThreshold: Optional[float] = None,
        verifyThreshold: Optional[float] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch identify handler with specified parameters.

        Args:
            estimateAttributes: whether to estimate basic attributes
            estimateEmotions: whether to estimate emotions
            estimateEthnicities: whether to estimate ethnicitites
            estimateQuality: whether to estimate quality
            estimateHeadPose: whether to estimate head pose
            extractExif: whether to extract exif
            yawThreshold: maximum deviation yaw angle
            rollThreshold: maximum deviation roll angle
            pitchThreshold: maximum deviation pitch angle
            scoreThreshold: score threshold
            verifyThreshold: verification threshold

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        data = {}
        if estimateAttributes is not None:
            data["estimate_attributes"] = estimateAttributes
        if estimateEmotions is not None:
            data["estimate_emotions"] = estimateEmotions
        if estimateEthnicities is not None:
            data["estimate_ethnicities"] = estimateEthnicities
        if estimateQuality is not None:
            data["estimate_quality"] = estimateQuality
        if estimateHeadPose is not None:
            data["estimate_head_pose"] = estimateHeadPose
        if extractExif is not None:
            data["extract_exif"] = extractExif
        if yawThreshold is not None:
            data["yaw_threshold"] = yawThreshold
        if rollThreshold is not None:
            data["roll_threshold"] = rollThreshold
        if pitchThreshold is not None:
            data["pitch_threshold"] = pitchThreshold
        if scoreThreshold is not None:
            data["score_threshold"] = scoreThreshold
        if verifyThreshold is not None:
            data["verify_threshold"] = verifyThreshold
        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(url=f"{self.baseUri}/handlers/verify", method="PATCH", body=payload, **kwargs)

    def verifyBatchRaw(
        self,
        candidates: List[BinaryReference],
        references: List[BinaryReference],
        noCache: Optional[bool] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher raw API allows to match each candidate with each reference descriptors.

        Args:
            candidates: list of BinaryReference objects for candidates
            references: list of BinaryReference objects for references

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching

        """
        queries = {"no_cache": noCache}
        data = {
            "candidates": [reference.asDict(encode=False) for reference in candidates],
            "references": [reference.asDict(encode=False) for reference in references],
        }
        payload = RequestPayload.buildMsgpack(body=data)
        return self.makeAuthRequest(
            url=f"{self.baseUri}/handlers/verify/raw",
            method="POST",
            queryParams=queries,
            body=payload.body,
            headers=payload.headers,
            **kwargs,
        )
