"""
Module realize attributes mixin
"""

from typing import Awaitable, List, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class AttributesMixin(MixinStub):
    """Attributes Mixin to work with attributes."""

    def extractAttrFromSample(
        self,
        sampleIds: list,
        scoreThreshold: float = 0,
        extractDescriptor: int = 1,
        extractBasicAttributes: int = 0,
        aggregateAttributes: int = 0,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract attribures from samples.

        Args:
            sampleIds: list of samples
            scoreThreshold: number [ 0 .. 1 ]; example: 0.99; descriptor quality score threshold.
                All the attributes with quality score below the threshold will be ignored (and not stored in the DB).
                The function will proceed as usual with all the remaining descriptors (if left).
            extractDescriptor: Available values : 0, 1; whether to extract face descriptor(s)
            extractBasicAttributes: Available values : 0, 1;
                whether to extract basic attributes (gender, age, ethnicity)
            aggregateAttributes: Available values : 0, 1; whether to aggregate face descriptor(s);
                If true, all extracted attributes will be aggregated and stored in as a single attribute.
                Otherwise, all attributes will be stored for every sample.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with attributes info

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.extractAttrFromSample(sampleIds=["ac3cc20a-439f-4f12-bd8a-f3b148fac51e"],
        ...                                         extractBasicAttributes=1, extractDescriptor=1)
        >>> print(response.json)
        [
          {
            "samples": [
              "ac3cc20a-439f-4f12-bd8a-f3b148fac51e"
            ],
            "url": "/1/attributes/cfce2aae-5747-4b20-b6ae-1a3159176546",
            "attribute_id": "cfce2aae-5747-4b20-b6ae-1a3159176546",
            "basic_attributes": {
              "age": 27,
              "gender": 0,
              "ethnicities": {
                "estimations": {
                  "african_american": 0,
                  "asian": 0,
                  "caucasian": 1,
                  "indian": 0
                },
                "predominant_ethnicity": "caucasian"
              }
            },
            "score": 0.9933221936
          }
        ]
        >>> print(response.statusCode)
        201

        """
        queries = {
            "score_threshold": scoreThreshold,
            "extract_descriptor": extractDescriptor,
            "extract_basic_attributes": extractBasicAttributes,
            "aggregate_attributes": aggregateAttributes,
        }
        payload = RequestPayload.buildJson(body=sampleIds)
        return self.makeRequest(f"{self.baseUri}/extractor", "POST", queryParams=queries, body=payload, **kwargs)

    def getAttributeIds(self, attributeIds: List[str], **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get attributes ids by filters

        Args:
            attributeIds: attribute ids

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with attributes info

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getAttributeIds(attributeIds=["74af13ae-f702-4ec9-acb0-e9432104451f",
        "02b77d66-60d3-47b3-af18-42faa6c867ba"])
        >>> print(response.json)
        [
          {'create_time': '2018-08-11T09:08:23.112Z',
           'attribute_id': '74af13ae-f702-4ec9-acb0-e9432104451f'},
          {'create_time': '2018-08-11T09:08:29.414Z',
           'attribute_id': '02b77d66-60d3-47b3-af18-42faa6c867ba'}
        ]
        >>> print(response.statusCode)
        200

        """
        queries = {"attribute_ids": attributeIds}
        return self.makeRequest(f"{self.baseUri}/attributes", "GET", queryParams=queries, **kwargs)

    def getAttributeIdsCount(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get attribute count

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: count of attributes

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getAttributeIdsCount(timeGte="2018-08-10T09:08:29.414Z")
        >>> print(response.text)

        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/attributes/count", "GET", **kwargs)

    def getBasicAttributesById(self, attributeId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get basic attributes (gender, age, ethnicity) by attrributes id.

        Args:
            attributeId: attribute Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with basic attributes

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getBasicAttributesById("7d60014f-d7c6-4398-a629-dc2616018d47")
        >>> print(response.json)
        {
         "basic_attributes": {
           "age": 20,
           "gender": 1,
           "ethnic_group": 2
         },
         "create_time": "2018-08-11T09:08:29.414Z"
        }
        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "GET", **kwargs)

    def checkBasicAttributesById(self, attributeId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist an attributes with id=attribute_id

        Args:
            attributeId: attribute Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.checkBasicAttributesById("7d60014f-d7c6-4398-a629-dc2616018d47")
        >>> print(response.text)

        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "HEAD", **kwargs)

    def deleteAttributeById(self, attributeId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete attributes by id. Associated face (if it exists) also will be deleted.

        Args:
            attributeId: attribute Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.deleteAttributeById("7d60014f-d7c6-4398-a629-dc2616018d47")
        >>> print(response.text)

        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "DELETE", **kwargs)

    def getAttributesSamples(self, attributeId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get samples which are associated with the attributes (gender, age, ethnicity, descriptor).

        Args:
            attributeId: attribute Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with basic attributes

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getAttributesSamples("7d60014f-d7c6-4398-a629-dc2616018d47")
        >>> print(response.json)
        [
            {
                "sample_id": "3ba4b5bc-b3cd-4a06-b6bf-254db69d63ed",
                "url": "/1/samples/3ba4b5bc-b3cd-4a06-b6bf-254db69d63ed"
            },
            {
                "sample_id": "946f92aa-90cf-4a64-aa58-b896ee1ab991",
                "url": "/1/samples/946f92aa-90cf-4a64-aa58-b896ee1ab991"
            }
        ]
        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}/samples", "GET", **kwargs)
