"""
Luna handlers client
"""

from numbers import Real
from typing import Any, Awaitable, Dict, List, Optional, Union

import aiohttp

from ..common import functions
from ..common.base_api import BaseHTTPAPI
from ..common.http_objs import (
    BinaryDescriptorData,
    BinaryImage,
    Policies,
    SampleForDetection,
    UrlForDetection,
    VerifierPolicies,
)
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload
from ..python_matcher.match_objects import BinaryReference
from .http_objs import EventAttributeUpgrade, FaceAttributeUpgrade, RawEvent, StreamEventsSource


class HandlersApi(BaseHTTPAPI):
    """
    Class for request to luna-handlers.

    """

    # multipart boundary; real property is defined above (BaseLunaHttpClient)
    multipartBoundary: str = "LUNACOREB0UnDaRyStRiNg"

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5090",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )

    def detectFaces(
        self,
        inputData: Union[BinaryImage, List[BinaryImage], List[UrlForDetection], List[str]],
        accountId: str,
        multifacePolicy: int = 1,
        estimateHeadPose: int = 0,
        detectLandmarks68: int = 0,
        extractExif: int = 0,
        estimateQuality: int = 0,
        estimateGaze: int = 0,
        estimateEyesAttributes: int = 0,
        estimateEmotions: int = 0,
        estimateMask: int = 0,
        estimateMouthAttributes: int = 0,
        pitchThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        warpedImage: int = 0,
        useExifInfo: Optional[int] = None,
        estimateFaceQuality: int = 0,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Detect faces on input images.

        Args:
            inputData: one of:
                        - Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
                        - list of urls with images
                        - list of images' ids
            accountId: account id
            multifacePolicy: multiple face detection policy:
                             0 - multiple face detection not allowed,
                             1 - multiple face detection allowed,
                             2 - get best detection from the image
            estimateHeadPose: Available values : 0, 1
            detectLandmarks68: Available values : 0, 1
            extractExif: Available values : 0, 1
            estimateQuality: Available values : 0, 1
            estimateGaze: Available values : 0, 1
            estimateEyesAttributes: Available values : 0, 1
            estimateMouthAttributes: Available values : 0, 1
            estimateEmotions: Available values : 0, 1
            estimateMask: Available values : 0, 1
            pitchThreshold: maximum deviation pitch angle from 0
            rollThreshold: maximum deviation roll angle from 0
            yawThreshold: maximum deviation yaw angle from 0
            warpedImage: Whether input image is a warped or arbitrary image.
            useExifInfo: whether to use exif info for auto orientation
            estimateFaceQuality: whether to estimate face quality

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with samples, exif, filename for succeeded images and
            errors for failed images will be returned.

        """
        queries = {
            "multiface_policy": multifacePolicy,
            "estimate_head_pose": estimateHeadPose,
            "detect_landmarks68": detectLandmarks68,
            "extract_exif": extractExif,
            "estimate_quality": estimateQuality,
            "estimate_gaze": estimateGaze,
            "estimate_eyes_attributes": estimateEyesAttributes,
            "estimate_mouth_attributes": estimateMouthAttributes,
            "estimate_emotions": estimateEmotions,
            "estimate_mask": estimateMask,
            "pitch_threshold": pitchThreshold,
            "roll_threshold": rollThreshold,
            "yaw_threshold": yawThreshold,
            "warped_image": warpedImage,
            "use_exif_info": useExifInfo,
            "estimate_face_quality": estimateFaceQuality,
        }

        payload = functions.createPayloadWithImages(inputData, {}, self.multipartBoundary)
        payload.applyAccountIdHeader(accountId)

        return self.makeRequest(f"{self.baseUri}/detector", "POST", queryParams=queries, body=payload, **kwargs)

    def extractAttrFromSample(
        self,
        accountId: str,
        sampleIds: List[str],
        scoreThreshold: float = 0,
        extractDescriptor: int = 1,
        extractBasicAttributes: int = 0,
        aggregateAttributes: int = 0,
        ttl: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract attributes from samples.

        Args:
            sampleIds: list of samples
            accountId: account id
            scoreThreshold: number [ 0 .. 1 ]; example: 0.99; descriptor quality score threshold.
                All the attributes with quality score below the threshold will be ignored (and not stored in the DB).
                The function will proceed as usual with all the remaining descriptors (if left).
            extractDescriptor: Available values : 0, 1; whether to extract face descriptor(s)
            extractBasicAttributes: Available values : 0, 1;
                whether to extract basic attributes (gender, age, ethnicity)
            aggregateAttributes: Available values : 0, 1; whether to aggregate face descriptor(s);
                If true, all extracted attributes will be aggregated and stored in as a single attribute.
                Otherwise, all attributes will be stored for every sample.
            ttl: temporary attribute ttl

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with attributes info

        """
        queries = {
            "score_threshold": scoreThreshold,
            "extract_descriptor": extractDescriptor,
            "extract_basic_attributes": extractBasicAttributes,
            "aggregate_attributes": aggregateAttributes,
            "ttl": ttl,
        }
        payload = RequestPayload.buildJson(body=sampleIds)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/extractor", "POST", queryParams=queries, body=payload, **kwargs)

    def upgradeObjectFromSample(
        self,
        data: Union[FaceAttributeUpgrade, EventAttributeUpgrade],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Upgrade basic attributes and/or descriptor in a object using samples.

        Args:
            data: face or attribute upgrade object
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with attributes info
        """
        payload = RequestPayload.buildJson(body=data.asDict())
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/extractor/upgrade", "POST", body=payload, **kwargs)

    def createHandler(
        self,
        accountId: str,
        policies: Optional[Policies] = None,
        handlerType: Optional[int] = None,
        lambdaId: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new handler. Handler determinate a list of rules for proccessing input images.
        You can set parameters for detecting faces, extracting attributes.
        You can set rules for matching attributes and creating faces from them.

        Args:
            policies: object with policies information
            handlerType: handler type (0 - static, 1 - dynamic, 2 - lambda)
            lambdaId: lambda id
            description: user description
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json new handler id and url

        """
        data = {}
        if description is not None:
            data["description"] = description
        if policies is not None:
            data["policies"] = policies.asDict()
        if handlerType is not None:
            data["handler_type"] = handlerType
        if lambdaId is not None:
            data["lambda_id"] = lambdaId
        payload = RequestPayload.buildJson(body=data)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/handlers", "POST", body=payload, **kwargs)

    def getHandlers(
        self,
        description: Optional[str] = None,
        accountId: Optional[str] = None,
        handlerType: Optional[int] = None,
        page: int = 1,
        pageSize: int = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get handlers by filters

        Args:
            description: Find all handlers with description which "like" of this parameter
            accountId: account id
            handlerType: handler type (0 - static, 1 - dynamic, 2 - lambda)
            page: The number of page
            pageSize: The numbers of items on page

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with handlers info

        """
        queries = {
            "account_id": accountId,
            "description": description,
            "page": page,
            "page_size": pageSize,
            "handler_type": handlerType,
        }
        return self.makeRequest(f"{self.baseUri}/handlers", "GET", queryParams=queries, **kwargs)

    def getHandlersCount(
        self,
        description: Optional[str] = None,
        handlerType: Optional[int] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get handlers count by filters

        Args:
            description: Find all handlers with description which "like" of this parameter
            accountId: account id
            handlerType: handler type (0 - static, 1 - dynamic, 2 - lambda)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`:  handlers count

        """
        queries = {"description": description, "account_id": accountId, "handler_type": handlerType}
        return self.makeRequest(f"{self.baseUri}/handlers/count", "GET", queryParams=queries, **kwargs)

    def getHandlerById(
        self, handlerId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get handler by its id

        Args:
            handlerId: handler id (uuid4)
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with handler info

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/handlers/{handlerId}", "GET", queryParams=queries, **kwargs)

    def updateHandler(
        self,
        handlerId: str,
        accountId: str,
        policies: Optional[Policies] = None,
        handlerType: Optional[int] = None,
        lambdaId: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update a handler. You can not update a part of handler.

        Args:
            handlerId: handler id
            policies: object with policies information
            handlerType: handler type (0 - static, 1 - dynamic, 2 - lambda)
            lambdaId: lambda id
            description: user description
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return nothing in text field

        """
        data = {}
        if description is not None:
            data["description"] = description
        if policies is not None:
            data["policies"] = policies.asDict()
        if handlerType is not None:
            data["handler_type"] = handlerType
        if lambdaId is not None:
            data["lambda_id"] = lambdaId
        payload = RequestPayload.buildJson(body=data)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/handlers/{handlerId}", "PUT", body=payload, **kwargs)

    def validateHandler(
        self,
        accountId: str,
        policies: Union[Policies, dict],
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Validate handler policies.

        Args:
            accountId: account id
            policies: object with policies information

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        data = {"policies": policies.asDict() if isinstance(policies, Policies) else policies}
        payload = RequestPayload.buildJson(body=data)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/handlers/validator", "POST", body=payload, **kwargs)

    def checkHandlerById(
        self, handlerId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check whether handler exists by its id

        Args:
            handlerId: handler id (uuid4)
            accountId: account id
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: no text returns

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/handlers/{handlerId}", "HEAD", queryParams=queries, **kwargs)

    def deleteHandlerById(
        self, handlerId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove handler by id

        Args:
            handlerId: handler id (uuid4)
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: no text returns

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/handlers/{handlerId}", "DELETE", queryParams=queries, **kwargs)

    def emitEvents(
        self,
        handlerId: str,
        accountId: str,
        inputData: Union[BinaryImage, List[BinaryImage], List[UrlForDetection], List[str], List[SampleForDetection]],
        policies: Optional[Union[Policies, dict]] = None,
        externalId: Optional[str] = None,
        userData: Optional[str] = None,
        aggregateAttributes: Optional[int] = None,
        source: Optional[str] = None,
        streamId: Optional[str] = None,
        city: Optional[str] = None,
        street: Optional[str] = None,
        district: Optional[str] = None,
        area: Optional[str] = None,
        houseNumber: Optional[str] = None,
        tags: Optional[str] = None,
        latitude: Optional[float] = None,
        longitude: Optional[float] = None,
        imageType: int = None,
        lunaEventTime: Optional[str] = None,
        lunaEventEndTime: Optional[str] = None,
        trackId: Optional[str] = None,
        useExifInfo: Optional[int] = None,
        meta: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Emit input events.

        Args:
            handlerId: handler id
            inputData: one of:
                        - Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
                        - list of urls with images
                        - list of images' ids
            policies: object with policies information for dynamic handler
            externalId: external id
            userData: user data
            aggregateAttributes: aggregate all attributes (descriptot, age, gender) or not (0 or 1)
            source: source
            streamId: stream id
            city: event city
            street: event street
            area: event area
            district: event district
            houseNumber: event house number
            tags: tags
            latitude: event latitude in degrees
            longitude: event longitude in degrees
            imageType: image type (0 - raw image, 1 - face warped image, 2 - body warped image)
            lunaEventTime: user defined event create time
            lunaEventEndTime: user defined event end time
            accountId: account id
            trackId: (str) event track id
            useExifInfo: whether to use exif info for auto orientation
            meta: user-defined event metadata

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json new handler id and url

        """
        queries = {
            "external_id": externalId,
            "user_data": userData,
            "aggregate_attributes": aggregateAttributes,
            "source": source,
            "stream_id": streamId,
            "tags": tags,
            "image_type": imageType,
            "city": city,
            "house_number": houseNumber,
            "street": street,
            "district": district,
            "area": area,
            "latitude": latitude,
            "longitude": longitude,
            "track_id": trackId,
            "use_exif_info": useExifInfo,
        }
        headers = {}
        if lunaEventTime is not None:
            headers["Luna-Event-Time"] = lunaEventTime
        if lunaEventEndTime is not None:
            headers["Luna-Event-End-Time"] = lunaEventEndTime
        payload = functions.createPayloadWithImages(inputData, headers, self.multipartBoundary, policies, meta)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(
            f"{self.baseUri}/handlers/{handlerId}/events", "POST", queryParams=queries, body=payload, **kwargs
        )

    def emitStreamEvents(
        self,
        handlerId: str,
        accountId: str,
        inputData: StreamEventsSource,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Emit stream events.

        Args:
            handlerId: handler id
            accountId: account id
            inputData: structure with stream event(s) source

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json new handler id and url

        """
        payload = RequestPayload.buildMsgpack(body=inputData.asDict())
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/handlers/{handlerId}/stream_events", "POST", body=payload, **kwargs)

    def saveEvent(
        self,
        accountId: str,
        handlerId: str,
        event: Union[RawEvent, Dict[str, Any]],
        waitSaving: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save raw event.

        Args:
            accountId: account id
            handlerId: handler id
            event: raw event
            waitSaving: whether to wait for event saving

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"wait_saving": waitSaving}
        payload = RequestPayload.buildJson(body=event.asDict() if isinstance(event, RawEvent) else event)
        payload.applyAccountIdHeader(accountId=accountId)
        return self.makeRequest(
            f"{self.baseUri}/handlers/{handlerId}/events/raw", "POST", queryParams=queries, body=payload, **kwargs
        )

    def createVerifier(
        self, accountId: str, policies: Optional[VerifierPolicies] = None, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new verifier. Verifier determinate a list of rules for processing & verification of input images.
        You can set parameters for detecting faces, extracting attributes.
        You can set rules for matching attributes and creating faces from them.

        Args:
            policies: object with policies information
            description: user description
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json new verifier id and url

        """
        data = {}
        if description is not None:
            data["description"] = description
        if policies is not None:
            data["policies"] = policies.asDict()
        payload = RequestPayload.buildJson(body=data)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/verifiers", "POST", body=payload, **kwargs)

    def getVerifier(
        self, verifierId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get verifier

        Args:
            verifierId: verifier id
            accountId: account id
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with verifier info

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/verifiers/{verifierId}", "GET", queryParams=queries, **kwargs)

    def putVerifier(
        self,
        verifierId: str,
        accountId: str,
        policies: Optional[VerifierPolicies] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Replace verifier

        Args:
            verifierId: verifier id
            policies: object with policies information
            description: user description
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: no text returns

        """
        data = {}
        if description is not None:
            data["description"] = description
        if policies is not None:
            data["policies"] = policies.asDict()
        payload = RequestPayload.buildJson(body=data)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/verifiers/{verifierId}", "PUT", body=payload, **kwargs)

    def checkVerifier(
        self, verifierId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check verifier existence

        Args:
            verifierId: verifier id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: no text returns

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/verifiers/{verifierId}", "HEAD", queryParams=queries, **kwargs)

    def deleteVerifier(
        self, verifierId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove verifier

         Args:
             verifierId: verifier id
             accountId: account id

         Returns:
             class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
             In body of :class: `~.LunaResponse`: no text returns

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/verifiers/{verifierId}", "DELETE", queryParams=queries, **kwargs)

    def getVerifiersCount(
        self, description: Optional[str] = None, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Count verifiers that satisfy the filters

        Args:
            description: verifier description ("like" parameter)
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`:  verifier count

        """
        queries = {"description": description, "account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/verifiers/count", "GET", queryParams=queries, **kwargs)

    def getVerifiers(
        self,
        description: Optional[str] = None,
        accountId: Optional[str] = None,
        page: int = 1,
        pageSize: int = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get verifiers by filters

        Args:
            description: verifier description ("like" parameter)
            accountId: account id
            page: The number of page
            pageSize: The numbers of items on page

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with verifiers info

        """
        queries = {"account_id": accountId, "description": description, "page": page, "page_size": pageSize}
        return self.makeRequest(f"{self.baseUri}/verifiers", "GET", queryParams=queries, **kwargs)

    def verifyRaw(
        self,
        candidates: List[BinaryReference],
        references: List[BinaryReference],
        verifierId: str,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Verify raw API allows to match each candidate with each reference descriptors.

        Args:
            verifierId: verifier id
            candidates: list of BinaryReference objects for candidates
            references: list of BinaryReference objects for references
                        verifierId: verifier id
            accountId: account id
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching

        """
        data = {
            "candidates": [reference.asDict(encode=False) for reference in candidates],
            "references": [reference.asDict(encode=False) for reference in references],
        }
        payload = RequestPayload.buildMsgpack(body=data)
        return self.makeRequest(
            f"{self.baseUri}/verifiers/{verifierId}/raw",
            "POST",
            queryParams={"account_id": accountId},
            body=payload,
            **kwargs,
        )

    def verifyData(
        self,
        verifierId: str,
        inputData: Union[BinaryImage, BinaryDescriptorData, List[Union[BinaryImage, BinaryDescriptorData]]],
        eventIds: Optional[List[str]] = None,
        externalIds: Optional[List[str]] = None,
        faceIds: Optional[List[str]] = None,
        attributeIds: Optional[List[str]] = None,
        imageType: Optional[int] = None,
        useExifInfo: Optional[int] = None,
        *,
        accountId,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Perform data verification against specified objects.
        Only one objects set is allowed.

        Args:
            verifierId: verifier id
            inputData: one of:
                        - Single image
                        - Single descriptor
                        - List of images and descriptors
            faceIds: faces ids to verify against
            eventIds: event ids to verify against
            externalIds: external ids to verify against
            attributeIds: temporary attribute ids to verify against
            imageType: image type (0 - raw image, 1 - face warped image)
            accountId: account id
            useExifInfo: whether to use exif info for auto orientation

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: json with verifications info

        """
        params = {
            "account_id": accountId,
            "face_ids": faceIds,
            "external_ids": externalIds,
            "event_ids": eventIds,
            "attribute_ids": attributeIds,
            "image_type": imageType,
            "use_exif_info": useExifInfo,
        }
        payload = functions.createPayloadWithImages(inputData, {}, self.multipartBoundary)

        return self.makeRequest(
            f"{self.baseUri}/verifiers/{verifierId}/verifications", "POST", queryParams=params, body=payload, **kwargs
        )
