"""
Luna-lambda client module
"""

from typing import Awaitable, List, Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from numbers import Real

import aiohttp

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload


class LambdaApi(BaseHTTPAPI):
    """
    Class for request to luna-lambda.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5210",
        api: Optional[int] = 3,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )

    def _getPayloadLambda(
        self,
        accountId: str,
        archive: bytes,
        lambdaName: Optional[str] = None,
        description: Optional[str] = None,
        lambdaType: Optional[Literal["standalone", "handlers", "tasks"]] = None,
        additionalUserDockerCommands: Optional[List[str]] = None,
        baseImageName: Optional[str] = None,
        baseImageTag: Optional[str] = None,
        workers: Optional[int] = None,
        enableGPU: Optional[bool] = None,
        podCount: Optional[int] = None,
    ) -> RequestPayload:
        """
        Get payload for create/validate lambda

        Args:
            accountId: account id
            archive: archive file bytes
            lambdaName: name of lambda
            description: description
            lambdaType: lambda type
            additionalUserDockerCommands: additional user docker commands
            baseImageName: base lambda image name
            baseImageTag: base lambda image tag (for custom image)
            workers: count of lambda workers
            enableGPU: whether to enable GPU
            podCount: count of lambda pods

        Returns:
            payload: RequestPayload
        """
        with aiohttp.MultipartWriter("form-data", self.multipartBoundary) as mpWriter:
            archivePart = mpWriter.append(archive, {"Content-Type": "application/zip"})
            archivePart.set_content_disposition(
                "attachment", name="archive", filename="archive.zip", quote_fields=False
            )

            creds = {"lambda_name": lambdaName}
            if description is not None:
                creds["description"] = description

            if lambdaName is not None:
                credsPart = mpWriter.append_json(creds)
                credsPart.set_content_disposition("form-data", name="credentials", quote_fields=False)

            kwargs = {}
            if lambdaType is not None:
                kwargs["lambda_type"] = lambdaType
            if additionalUserDockerCommands is not None:
                kwargs["additional_user_docker_commands"] = additionalUserDockerCommands
            if baseImageName is not None:
                kwargs["base_image"] = {"name": baseImageName}
                if baseImageTag is not None:
                    kwargs["base_image"]["tag"] = baseImageTag
            if workers is not None:
                kwargs["workers"] = workers
            if kwargs:
                parametersPart = mpWriter.append_json(kwargs)
                parametersPart.set_content_disposition("form-data", name="parameters", quote_fields=False)
            kwargs = {}
            if enableGPU is not None:
                kwargs["enable_gpu"] = int(enableGPU)
            if podCount is not None:
                kwargs["pod_count"] = podCount
            if kwargs:
                deployParametersPart = mpWriter.append_json(kwargs)
                deployParametersPart.set_content_disposition("form-data", name="deploy_parameters", quote_fields=False)
            payload = RequestPayload.buildMultipart(body=mpWriter)

        payload.applyAccountIdHeader(accountId)
        return payload

    def createLambda(
        self,
        accountId: str,
        archive: bytes,
        lambdaName: str,
        description: Optional[str] = None,
        lambdaType: Optional[Literal["standalone", "handlers", "tasks"]] = None,
        additionalUserDockerCommands: Optional[List[str]] = None,
        baseImageName: Optional[str] = None,
        baseImageTag: Optional[str] = None,
        workers: Optional[int] = None,
        enableGPU: Optional[bool] = None,
        podCount: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create lambda

        Args:
            accountId: account id
            archive: archive file bytes
            lambdaName: name of lambda
            description: lambda description
            lambdaType: lambda type
            additionalUserDockerCommands: additional user docker commands
            baseImageName: base lambda image name
            baseImageTag: base lambda image tag (for custom image)
            workers: count of lambda workers
            enableGPU: whether to enable GPU
            podCount: count of lambda pods

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        payload = self._getPayloadLambda(
            accountId=accountId,
            archive=archive,
            description=description,
            lambdaName=lambdaName,
            lambdaType=lambdaType,
            additionalUserDockerCommands=additionalUserDockerCommands,
            baseImageName=baseImageName,
            baseImageTag=baseImageTag,
            workers=workers,
            enableGPU=enableGPU,
            podCount=podCount,
        )
        return self.makeRequest(f"{self.baseUri}/lambdas", "POST", body=payload, **kwargs)

    def patchLambda(
        self,
        lambdaId: str,
        accountId: str,
        lambdaName: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            lambdaName: name of lambda
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        jsonBody = {}
        if lambdaName is not None:
            jsonBody["lambda_name"] = lambdaName
        if description is not None:
            jsonBody["description"] = description
        payload = RequestPayload.buildJson(body=jsonBody)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}", "PATCH", body=payload, **kwargs)

    def updateLambda(
        self,
        lambdaId: str,
        accountId: str,
        archive: bytes,
        lambdaName: str,
        description: Optional[str] = None,
        lambdaType: Optional[Literal["standalone", "handlers", "tasks"]] = None,
        additionalUserDockerCommands: Optional[List[str]] = None,
        baseImageName: Optional[str] = None,
        baseImageTag: Optional[str] = None,
        workers: Optional[int] = None,
        enableGPU: Optional[bool] = None,
        podCount: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            archive: archive file bytes
            lambdaName: name of lambda
            description: description
            lambdaType: lambda type
            additionalUserDockerCommands: additional user docker commands
            baseImageName: base lambda image name
            baseImageTag: base lambda image tag (for custom image)
            workers: count of lambda workers
            enableGPU: whether to enable GPU
            podCount: count of lambda pods

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        payload = self._getPayloadLambda(
            accountId=accountId,
            archive=archive,
            description=description,
            lambdaName=lambdaName,
            lambdaType=lambdaType,
            additionalUserDockerCommands=additionalUserDockerCommands,
            baseImageName=baseImageName,
            baseImageTag=baseImageTag,
            workers=workers,
            enableGPU=enableGPU,
            podCount=podCount,
        )
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}", "PUT", body=payload, **kwargs)

    def updateLambdaImage(
        self,
        lambdaId: str,
        accountId: str,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update lambda using latest base image

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestHeaders = {"Luna-Account-Id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}/update", "POST", headers=requestHeaders, **kwargs)

    def getLambdas(
        self,
        accountId: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        lambdaType: Optional[Literal["standalone", "handlers", "tasks"]] = None,
        lambdaNameFilter: Optional[str] = None,
        page: int = 1,
        pageSize: int = 100,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lambdas by filters.

        Args:
            accountId: account id
            createTimeLt: upper bound of face create time
            createTimeGte: lower bound of face create time
            lambdaNameFilter: lambda name filter
            lambdaType: lambda type
            page: pagination page value
            pageSize: pagination page size value

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambdas will be returned.

        """
        queries = {
            "account_id": accountId,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "lambda_type": lambdaType,
            "lambda_name": lambdaNameFilter,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/lambdas", "GET", queryParams=queries, **kwargs)

    def getLambda(
        self, lambdaId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lambda by id.

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}", "GET", queryParams=queries, **kwargs)

    def checkLambda(
        self, lambdaId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check lambda existence by id.

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}", "HEAD", queryParams=queries, **kwargs)

    def deleteLambda(
        self, lambdaId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete lambda by id.

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}", "DELETE", queryParams=queries, **kwargs)

    def validateLambda(
        self,
        accountId: str,
        archive: bytes,
        lambdaType: Optional[Literal["standalone", "handlers", "tasks"]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Validate lambda.

        Args:
            accountId: account id
            archive: archive file bytes
            lambdaType: lambda type

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        payload = self._getPayloadLambda(accountId=accountId, archive=archive, lambdaType=lambdaType)
        return self.makeRequest(f"{self.baseUri}/lambdas/validator", "POST", body=payload, **kwargs)

    def getLambdaStatus(
        self, lambdaId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check lambda status by id.

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda status will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}/status", "GET", queryParams=queries, **kwargs)

    def getLambdaLogs(
        self, lambdaId: str, tailLines: int = None, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lambda logs

        Args:
            lambdaId: lambda id
            tailLines: number of last string from lambda logs
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda status will be returned.
        """
        queries = {"tail_lines": tailLines, "account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}/logs", "GET", queryParams=queries, **kwargs)

    def getLambdaImageCreationStatus(
        self, lambdaId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check lambda image creation status by id.

        Args:
            lambdaId: lambda id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda status will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}/image/status", "GET", queryParams=queries, **kwargs)

    def getLambdaImageCreationLogs(
        self, lambdaId: str, tailLines: int = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lambda image creation logs

        Args:
            lambdaId: lambda id
            tailLines: number of last string from lambda logs

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda status will be returned.
        """
        queries = {"tail_lines": tailLines}
        return self.makeRequest(f"{self.baseUri}/lambdas/{lambdaId}/image/logs", "GET", queryParams=queries, **kwargs)

    def getLambdaSpec(
        self,
        lambdaId: str,
        accept: str = "text/html",
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lambda spec

        Args:
            lambdaId: lambda id
            accept: accept header
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        queries = {"account_id": accountId}
        headers = {"Accept": accept}
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/docs/spec", method="GET", headers=headers, queryParams=queries, **kwargs
        )

    def proxyLambdaPost(
        self,
        lambdaId: str,
        accountId: str,
        path: str,
        body: Optional[Union[str, bytes, dict, list, aiohttp.Payload, RequestPayload]] = None,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy POST request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            body: request body
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestHeaders = {"Luna-Account-Id": accountId}
        if headers is not None:
            requestHeaders.update(headers)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="POST",
            body=body,
            headers=requestHeaders,
            queryParams=queries,
            **kwargs,
        )

    def proxyLambdaPut(
        self,
        lambdaId: str,
        accountId: str,
        path: str,
        body: Optional[Union[str, bytes, dict, list, aiohttp.Payload, RequestPayload]] = None,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy PUT request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            body: request body
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestHeaders = {"Luna-Account-Id": accountId}
        if headers is not None:
            requestHeaders.update(headers)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="PUT",
            body=body,
            headers=requestHeaders,
            queryParams=queries,
            **kwargs,
        )

    def proxyLambdaPatch(
        self,
        lambdaId: str,
        path: str,
        accountId: Optional[str] = None,
        body: Optional[Union[str, bytes, dict, list, aiohttp.Payload, RequestPayload]] = None,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy PATCH request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            body: request body
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestQueries = {"account_id": accountId}
        if queries is not None:
            requestQueries.update(queries)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="PATCH",
            body=body,
            headers=headers,
            queryParams=requestQueries,
            **kwargs,
        )

    def proxyLambdaGet(
        self,
        lambdaId: str,
        path: str,
        accountId: Optional[str] = None,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy GET request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestQueries = {"account_id": accountId}
        if queries is not None:
            requestQueries.update(queries)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="GET",
            headers=headers,
            queryParams=requestQueries,
            **kwargs,
        )

    def proxyLambdaHead(
        self,
        lambdaId: str,
        path: str,
        accountId: Optional[str] = None,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy HEAD request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestQueries = {"account_id": accountId}
        if queries is not None:
            requestQueries.update(queries)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="HEAD",
            headers=headers,
            queryParams=requestQueries,
            **kwargs,
        )

    def proxyLambdaDelete(
        self,
        lambdaId: str,
        accountId: str,
        path: str,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy DELETE request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestQueries = {"account_id": accountId}
        if queries is not None:
            requestQueries.update(queries)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="DELETE",
            headers=headers,
            queryParams=requestQueries,
            **kwargs,
        )

    def proxyLambdaOptions(
        self,
        lambdaId: str,
        accountId: str,
        path: str,
        queries: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Proxy OPTIONS request to lambda

        Args:
            lambdaId: lambda id
            accountId: account id
            path: lambda uri path
            headers: request headers
            queries: request queries

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lambda_id will be returned.
        """
        requestQueries = {"account_id": accountId}
        if queries is not None:
            requestQueries.update(queries)
        return self.makeRequest(
            f"{self.baseUri}/lambdas/{lambdaId}/proxy/{path}",
            method="OPTIONS",
            headers=headers,
            queryParams=requestQueries,
            **kwargs,
        )
