"""
Module realize attributes mixin
"""

from typing import Awaitable, Optional, Union

import aiohttp

from luna3.common.http_objs import RawDescriptor
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.common.utils import getPreparedDescriptors
from luna3.lunavl.mixins.base import MixinStub


class AttributesMixin(MixinStub):
    """Attributes Mixin to work with attributes."""

    def extractAttrFromSample(
        self,
        sampleIds: list,
        scoreThreshold: float = 0,
        extractDescriptor: int = 1,
        extractBasicAttributes: int = 0,
        aggregateAttributes: int = 0,
        ttl: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Extract attributes from samples.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/extractAttributes

        Args:
            sampleIds: list of samples
            scoreThreshold: number [ 0 .. 1 ]; example: 0.99; descriptor quality score threshold.
                All the attributes with quality score below the threshold will be ignored (and not stored in the DB).
                The function will proceed as usual with all the remaining descriptors (if left).
            extractDescriptor: Available values : 0, 1; whether to extract face descriptor(s)
            extractBasicAttributes: Available values : 0, 1;
                whether to extract basic attributes (gender, age, ethnicity)
            aggregateAttributes: Available values : 0, 1; whether to aggregate face descriptor(s);
                If true, all extracted attributes will be aggregated and stored in as a single attribute.
                Otherwise, all attributes will be stored for every sample.
            ttl: temporary attribute ttl

        Returns:
            LunaResponse: body contains json with attributes info.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> response = client.extractAttrFromSample(
        ...     sampleIds=[sampleId], extractBasicAttributes=1, extractDescriptor=1,
        ... )
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        [
            {
                "attribute_id": ...,
                "basic_attributes": {
                    "age": 26,
                    "ethnicities": {
                        "estimations": {
                            "african_american": ...,
                            "asian": ...,
                            "caucasian": ...,
                            "indian": ...
                        },
                        "predominant_ethnicity": "caucasian"
                    },
                    "gender": 0
                },
                "external_url": "http://127.0.0.1:5000/6/attributes/...",
                "samples": [...],
                "score": ...,
                "url": "/6/attributes/..."
            }
        ]

        """
        queries = {
            "score_threshold": scoreThreshold,
            "extract_descriptor": extractDescriptor,
            "extract_basic_attributes": extractBasicAttributes,
            "aggregate_attributes": aggregateAttributes,
            "ttl": ttl,
        }
        payload = RequestPayload.buildJson(body=sampleIds)
        return self.makeRequest(f"{self.baseUri}/extractor", "POST", queryParams=queries, body=payload, **kwargs)

    def getAttributes(
        self,
        attributeIds: list[str],
        descriptorVersion: int = None,
        targets: Optional[list[str]] = None,
        accountIdFilter: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get attributes ids by filters

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getAttributes

        Args:
            attributeIds: list of attributes of interest
            targets: Available targets: `descriptor`, `basic_attributes`, `basic_attributes_samples`,
                `face_descriptors_samples`, `account_id`, `create_time`
            descriptorVersion: descriptor version to receive
            accountIdFilter: account id to get attributes

        Returns:
            LunaResponse: body contains json with attributes info.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP, DESCRIPTOR
        >>>
        >>> sampleId = client.detectFaces(
        ...     inputData=FACE_WARP, raiseError=True
        ... ).json["images"][0]["detections"]["samples"][0]["face"]["sample_id"]
        >>> basicAttributes = {'ethnicity': 3, 'age': 26, 'gender': 0}
        >>> attributeId = client.createAttribute(
        ...     basicAttributes = basicAttributes, basicAttributesSamples=[sampleId], descriptorSamples=[sampleId],
        ...                       descriptors=[DESCRIPTOR], raiseError=True
        ... ).json["attribute_id"]
        >>> response = client.getAttributes(
        ...     attributeIds=[attributeId], targets=["basic_attributes_samples", "account_id"]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True))  # doctest: +ELLIPSIS
        {"attributes": [{"account_id": ..., "basic_attributes_samples": [...]}]}

        """
        queries = {
            "attribute_ids": attributeIds,
            "targets": targets,
            "descriptor_version": descriptorVersion,
            "account_id": accountIdFilter,
        }
        return self.makeRequest(f"{self.baseUri}/attributes", "GET", queryParams=queries, **kwargs)

    def createAttribute(
        self,
        basicAttributes: Optional[dict] = None,
        descriptors: Optional[list[Union[bytes, RawDescriptor]]] = None,
        descriptorSamples: list[str] = None,
        basicAttributesSamples: list[str] = None,
        xpk: bytes = None,
        ttl: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new attributes. Notice that XPK argument overwrites basic attributes and descriptors above.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/createAttributes

        Args:
            descriptors: list of descriptors different versions
            descriptorSamples: list of warp image id from which the attribute was extracted.
            basicAttributes: dict with basic attributes (age, gender, ethnicity)
            basicAttributesSamples: list of warp image id from which the basic attribute was extracted.
            xpk: SDKObject bytes to load attributes from
            ttl: temporary attribute ttl

        Returns:
            LunaResponse: body contains json with sample_id and url.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP, DESCRIPTOR
        >>>
        >>> sampleId = client.detectFaces(
        ...     inputData=FACE_WARP, raiseError=True
        ... ).json["images"][0]["detections"]["samples"][0]["face"]["sample_id"]
        >>> basicAttributes = {'ethnicity': 3, 'age': 26, 'gender': 0}
        >>> response = client.createAttribute(
        ...     basicAttributes = basicAttributes, basicAttributesSamples=[sampleId], descriptorSamples=[sampleId],
        ...     descriptors=[DESCRIPTOR],
        ... )
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True)) # doctest: +ELLIPSIS
        {"attribute_id": ..., "url": ...}

        """
        meta = {}
        if descriptorSamples is not None:
            meta["face_descriptor_samples"] = descriptorSamples
        if basicAttributesSamples is not None:
            meta["basic_attributes_samples"] = basicAttributesSamples
        if xpk is not None:
            with aiohttp.MultipartWriter("form-data", self.multipartBoundary) as mpWriter:
                xpkPart = mpWriter.append(xpk, {"Content-Type": "application/x-vl-xpk"})
                xpkPart.set_content_disposition("attachment", name="xpk_file", filename="xpk-file", quote_fields=False)
                metaPart = mpWriter.append_json(meta, {"Content-Type": "application/json"})
                metaPart.set_content_disposition("attachment", name="meta", filename="meta", quote_fields=False)

                payload = RequestPayload.buildMultipart(body=mpWriter)
        else:
            body = meta
            if basicAttributes is not None:
                body["basic_attributes"] = basicAttributes
            if descriptors is not None:
                body["face_descriptors"] = getPreparedDescriptors(descriptors)

            payload = RequestPayload.buildJson(body=body)

        query = {"ttl": ttl}
        return self.makeRequest(f"{self.baseUri}/attributes", "POST", queryParams=query, body=payload, **kwargs)

    def getBasicAttributesById(
        self,
        attributeId: str,
        descriptorVersion: int = None,
        targets: Optional[list[str]] = None,
        accountIdFilter: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get basic attributes (gender, age, ethnicity) by attrribute id.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getAttribute

        Args:
            attributeId: attribute Id
            descriptorVersion: descriptor version to receive
            targets: Available targets: `descriptor`, `basic_attributes`, `basic_attributes_samples`,
                `face_descriptors_samples`, `account_id`, `create_time`
            accountIdFilter: account id to get attribute

        Returns:
            LunaResponse: body contains json with basic attributes.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(
        ...     sampleIds=[sampleId], extractBasicAttributes=1, extractDescriptor=1, raiseError=True,
        ... ).json[0]["attribute_id"]
        >>> response = client.getBasicAttributesById(
        ...     attributeId, targets=["create_time", "attribute_id", "account_id", "face_descriptor_samples",
        ...                           "basic_attributes_samples", "face_descriptor", "basic_attributes"]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "attribute_id": ...,
            "basic_attributes": {
                "age": ...,
                "ethnicity": ...,
                "gender": ...
            },
            "basic_attributes_samples": [...],
            "create_time": ...,
            "face_descriptor": ...,
            "face_descriptor_samples": [...]
        }

        """
        queries = {"targets": targets, "descriptor_version": descriptorVersion, "account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "GET", queryParams=queries, **kwargs)

    def checkBasicAttributesById(
        self, attributeId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist an attributes with id=attribute_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkAttribute

        Args:
            attributeId: attribute Id
            accountIdFilter: account id to check attribute

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(sampleIds=[sampleId], raiseError=True).json[0]["attribute_id"]
        >>> response = client.checkBasicAttributesById(attributeId=attributeId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "HEAD", queryParams=queries, **kwargs)

    def deleteAttributeById(self, attributeId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete attributes by id. Associated face (if it exists) also will be deleted.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteAttribute

        Args:
            attributeId: attribute Id

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(sampleIds=[sampleId], raiseError=True).json[0]["attribute_id"]
        >>> response = client.deleteAttributeById(attributeId=attributeId)
        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/attributes/{attributeId}", "DELETE", **kwargs)

    def getAttributeSamples(
        self, attributeId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Retrieve a attribute by id.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkAttribute

        Args:
            attributeId: attribute id
            accountIdFilter: account id to get attribute samples

        Returns:
            LunaResponse: body contains json with samples.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(sampleIds=[sampleId], raiseError=True).json[0]["attribute_id"]
        >>> response = client.getAttributeSamples(attributeId=attributeId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True))  # doctest: +ELLIPSIS
        {"samples": [...]}

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(
            f"{self.baseUri}/attributes/{attributeId}/samples", "GET", queryParams=queries, **kwargs
        )
