"""
Module realize objects mixin
"""

from typing import Awaitable, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class ObjectsMixin(MixinStub):
    """Objects Mixin to work with objects."""

    def saveObject(
        self, objectBody: str, contentType: Optional[str] = "application/json", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save object to the system.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/createObjects

        Args:
            objectBody: object, available: text, json
            contentType: content-type of object or application/json for default

        Returns:
            LunaResponse: body contains json with object_id and url.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> response = client.saveObject("1,2,3", contentType = "text/plain")
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True)) # doctest: +ELLIPSIS
        {"external_url": "http://127.0.0.1:5000/6/objects/...", "object_id": ..., "url": ...}

        """
        payload = RequestPayload(body=objectBody, headers={"Content-Type": contentType})
        return self.makeRequest(f"{self.baseUri}/objects", "POST", body=payload, **kwargs)

    def getObject(
        self, objectId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get object by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getObject

        Args:
            objectId: object id
            accountIdFilter: account id to get object

        Returns:
            LunaResponse: body contains object or string.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> objectBody = "1,2,3"
        >>> contentType = "text/plain"
        >>> objectId = client.saveObject(objectBody=objectBody, contentType=contentType, raiseError=True).json[
        ...     "object_id"
        ... ]
        >>> response = client.getObject(objectId=objectId)
        >>> print(response.statusCode)
        200
        >>> print(response.text)
        1,2,3

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/objects/{objectId}", "GET", queryParams=queries, **kwargs)

    def checkObject(
        self, objectId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist an object with id=object_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkObject

        Args:
            objectId: object id
            accountIdFilter: account id to check object

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> objectBody = "1,2,3"
        >>> contentType = "text/plain"
        >>> objectId = client.saveObject(objectBody=objectBody, contentType=contentType, raiseError=True).json[
        ...     "object_id"
        ... ]
        >>> response = client.checkObject(objectId=objectId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/objects/{objectId}", "HEAD", queryParams=queries, **kwargs)

    def deleteObject(self, objectId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete object with id=object_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteObject

        Args:
            objectId: object id

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> objectBody = "1,2,3"
        >>> contentType = "text/plain"
        >>> objectId = client.saveObject(
        ...     objectBody=objectBody, contentType=contentType, raiseError=True
        ... ).json["object_id"]
        >>> response = client.deleteObject(objectId=objectId)
        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/objects/{objectId}", "DELETE", **kwargs)
