"""
Module implements client for luna-python-matcher
"""

from numbers import Real
from typing import Awaitable, List, Optional, Union

import aiohttp

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload
from ..python_matcher.match_objects import (
    AttributeFilters,
    BinaryReference,
    Candidates,
    EventFilters,
    FaceFilters,
    Reference,
)


class PythonMatcherApi(BaseHTTPAPI):
    """
    Class for request to luna-python-matcher.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5100",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )

    def matchRaw(
        self,
        candidates: List[BinaryReference],
        references: List[BinaryReference],
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher raw API allows to match each candidate with each reference descriptors.

        Args:
            candidates: list of BinaryReference objects for candidates
            references: list of BinaryReference objects for references
            accept: expected content type

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching for each reference

        """
        jsonData = {
            "candidates": [reference.asDict(encode=False) for reference in candidates],
            "references": [reference.asDict(encode=False) for reference in references],
        }
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(f"{self.baseUri}/matcher/raw", "POST", body=payload, accept=accept, **kwargs)

    def matchFaces(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountId: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.


        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, attributes, faces, raw descriptors)
            accountId: account id
            accept: expected content type

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        """
        queries = {"account_id": accountId}
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/faces", "POST", queryParams=queries, body=payload, accept=accept, **kwargs
        )

    def matchBodies(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountId: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.


        Args:
            candidates: Candidates object with info about candidates
                        (possible type variants: events)
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, raw descriptors)
            accountId: account id
            accept: expected content type

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        """
        queries = {"account_id": accountId}
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/bodies", "POST", queryParams=queries, body=payload, accept=accept, **kwargs
        )

    def crossmatchFaces(
        self,
        candidateFilters: Union[FaceFilters, EventFilters, AttributeFilters],
        referenceFilters: Union[FaceFilters, EventFilters, AttributeFilters],
        limit: Optional[int] = None,
        threshold: Optional[int] = None,
        sorting: Optional[int] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Cross Matcher API allows to submit tasks to a service that searches for faces/events by given filters and
        matches them with each other. Resource implements face descriptors matching.

        Args:
            candidateFilters: candidate filters
            referenceFilters: reference filters
            limit: matching limit
            threshold: matching threshold
            sorting: whether to sort match results
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        """
        queries = {"account_id": accountId, "sorting": sorting}
        jsonData = {
            "candidates": candidateFilters.asNoneEmptyDict(),
            "references": referenceFilters.asNoneEmptyDict(),
        }
        if limit is not None:
            jsonData["limit"] = limit
        if threshold is not None:
            jsonData["threshold"] = threshold
        payload = RequestPayload.buildJson(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/crossmatcher/faces", "POST", queryParams=queries, body=payload, **kwargs
        )

    def crossmatchBodies(
        self,
        candidateFilters: EventFilters,
        referenceFilters: EventFilters,
        limit: Optional[int] = None,
        threshold: Optional[int] = None,
        sorting: Optional[int] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Cross Matcher API allows to submit tasks to a service that searches for events by given filters and
        matches them with each other. Resource implements body descriptors matching.

        Args:
            candidateFilters: candidate events filters
            referenceFilters: reference events filters
            limit: matching limit
            threshold: matching threshold
            sorting: whether to sort match results
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        """
        queries = {"account_id": accountId, "sorting": sorting}
        jsonData = {
            "candidates": candidateFilters.asNoneEmptyDict(),
            "references": referenceFilters.asNoneEmptyDict(),
        }
        if limit is not None:
            jsonData["limit"] = limit
        if threshold is not None:
            jsonData["threshold"] = threshold
        payload = RequestPayload.buildJson(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/crossmatcher/bodies", "POST", queryParams=queries, body=payload, **kwargs
        )
