"""Module contains utils for tasks"""

from typing import Optional

from luna3.common.requests import RequestPayload
from luna3.tasks.http_objs import TaskNotificationPolicy


def getGCTaskContent(**kwargs) -> dict:
    """
    Get GC task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    target = kwargs.get("target")
    taskContent = {"target": target, "filters": {}}

    handlerId = kwargs.get("handlerId")
    createTimeLt = kwargs.get("createTimeLt")
    insertTimeLt = kwargs.get("insertTimeLt")
    gcAccountId = kwargs.get("gcAccountId")
    createTimeGte = kwargs.get("createTimeGte")
    userData = kwargs.get("userData")
    listId = kwargs.get("listId")
    descriptorVersion = kwargs.get("descriptorVersion")
    descriptorType = kwargs.get("descriptorType")
    storeResults = kwargs.get("storeResults")
    removeSamples = kwargs.get("removeSamples")
    removeImageOrigins = kwargs.get("removeImageOrigins")

    if handlerId is not None:
        taskContent["filters"]["handler_id"] = handlerId
    if createTimeLt is not None:
        taskContent["filters"]["create_time__lt"] = createTimeLt
    if insertTimeLt is not None and target == "events":
        taskContent["filters"]["insert_time__lt"] = insertTimeLt
    if gcAccountId is not None and target in ("events", "faces"):
        taskContent["filters"]["account_id"] = gcAccountId
    if createTimeGte is not None and target == "faces":
        taskContent["filters"]["create_time__gte"] = createTimeGte
    if userData is not None and target == "faces":
        taskContent["filters"]["user_data"] = userData
    if listId is not None and target == "faces":
        taskContent["filters"]["list_id"] = listId
    if descriptorVersion is not None:
        taskContent["filters"]["descriptor_version"] = descriptorVersion
    if descriptorType is not None:
        taskContent["filters"]["descriptor_type"] = descriptorType
    if storeResults is not None:
        taskContent["store_results"] = storeResults
    if removeSamples is not None:
        taskContent["remove_samples"] = removeSamples
    if removeImageOrigins is not None:
        taskContent["remove_image_origins"] = removeImageOrigins

    return taskContent


def getLinkerTaskContent(**kwargs) -> dict:
    """
    Get linker task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    filters = kwargs.get("filters")
    filtersDict = filters.asNoneEmptyDict() if hasattr(filters, "asNoneEmptyDict") else filters

    taskContent = {"objects_type": filtersDict.pop("origin"), "filters": filtersDict}
    listId = kwargs.get("listId")
    createList = kwargs.get("createList")
    listUserData = kwargs.get("listUserData")

    if listId is not None:
        taskContent["list_id"] = listId
    if createList is not None:
        taskContent["create_list"] = createList
    if listUserData is not None:
        taskContent["user_data"] = listUserData

    return taskContent


def getClusteringTaskContent(**kwargs) -> dict:
    """
    Get clustering task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    filters = kwargs.get("filters")
    useTrackInfo = kwargs.get("useTrackInfo")
    descriptorTypeFilter = kwargs.get("descriptorTypeFilter")
    threshold = kwargs.get("threshold")
    description = kwargs.get("description")
    limit = kwargs.get("limit")

    filtersDict = filters.asNoneEmptyDict() if hasattr(filters, "asNoneEmptyDict") else filters

    taskContent = {
        "filters": filtersDict,
        "objects_type": filtersDict.pop("origin"),
    }
    if useTrackInfo is not None:
        taskContent.setdefault("params", {})["use_track_info"] = useTrackInfo
    if threshold is not None:
        taskContent["threshold"] = threshold
    if descriptorTypeFilter is not None:
        taskContent["descriptor"] = {"type": descriptorTypeFilter}
    if description is not None:
        taskContent["description"] = description
    if limit is not None:
        taskContent["limit"] = limit

    return taskContent


def getExporterTaskContent(**kwargs) -> dict:
    """
    Get exporter task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    filters = kwargs.get("filters")
    columns = kwargs.get("columns")
    saveImages = kwargs.get("saveImages")
    csvDelimiter = kwargs.get("csvDelimiter")
    description = kwargs.get("description")
    descriptorTypeFilter = kwargs.get("descriptorTypeFilter")

    filtersDict = filters.asNoneEmptyDict() if hasattr(filters, "asNoneEmptyDict") else filters

    taskContent = {
        "csv_delimiter": csvDelimiter,
        "objects_type": filtersDict.pop("origin"),
        "filters": filtersDict,
    }
    if columns is not None:
        taskContent["columns"] = columns
    if description is not None:
        taskContent["description"] = description
    if saveImages is not None:
        taskContent["save_images"] = saveImages
    if descriptorTypeFilter is not None:
        taskContent["descriptor"] = {"type": descriptorTypeFilter}

    return taskContent


def getAETaskContent(**kwargs) -> dict:
    """
    Get additional extract task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    taskContent = {}
    options = {}
    extractionTarget = kwargs.get("extraction_target")
    missing = kwargs.get("missing")
    filters = kwargs.get("filters")
    descriptorVersion = kwargs.get("descriptor_version")
    if extractionTarget is not None:
        taskContent["extraction_target"] = extractionTarget
    if missing is not None:
        options["missing"] = missing
    if filters is not None:
        taskContent["filters"] = filters.asDict()
    if descriptorVersion is not None:
        options["descriptor_version"] = descriptorVersion
    if options:
        taskContent["options"] = options
    return taskContent


def getCMTaskContent(**kwargs) -> dict:
    """
    Get cross matching task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    referenceFilters = kwargs.get("referenceFilters")
    candidateFilters = kwargs.get("candidateFilters")
    accountIdFilter = kwargs.get("accountIdFilter")
    threshold = kwargs.get("threshold")
    limit = kwargs.get("limit")
    descriptorTypeFilter = kwargs.get("descriptorTypeFilter")
    referenceFiltersDict = referenceFilters.asNoneEmptyDict()
    candidateFiltersDict = candidateFilters.asNoneEmptyDict()
    taskContent = {
        "filters": {
            "reference_type": referenceFiltersDict.pop("origin"),
            "reference_filters": referenceFiltersDict,
            "candidate_type": candidateFiltersDict.pop("origin"),
            "candidate_filters": candidateFiltersDict,
        }
    }
    if accountIdFilter is not None:
        taskContent["filters"]["account_id"] = accountIdFilter
    if threshold is not None:
        taskContent["threshold"] = threshold
    if limit is not None:
        taskContent["limit"] = limit
    if descriptorTypeFilter is not None:
        taskContent["filters"]["descriptor"] = {"type": descriptorTypeFilter}
    return taskContent


def getROCTaskContent(**kwargs) -> dict:
    """
    Get roc task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    markup = kwargs.get("markup")
    accountIdFilter = kwargs.get("accountIdFilter")
    thresholdHitTop = kwargs.get("thresholdHitTop")
    limit = kwargs.get("limit")
    keyFPRs = kwargs.get("keyFPRs")
    taskContent = {"markup": markup, "filters": {}}
    if accountIdFilter is not None:
        taskContent["filters"]["account_id"] = accountIdFilter
    if thresholdHitTop is not None:
        taskContent["threshold_hit_top"] = thresholdHitTop
    if limit is not None:
        taskContent["limit"] = limit
    if keyFPRs is not None:
        taskContent["key_FPRs"] = keyFPRs
    return taskContent


def getEstimatorTaskContent(withoutAccountFilters: bool = False, **kwargs) -> dict:
    """
    Get estimator task content as dict
    Args:
        withoutAccountFilters: whether account id filter is not allowed (for luna-api)
    Keyword Args:
        kwargs from user request

    Returns:
        dict with content
    """
    taskContent = {}
    handler = kwargs.get("handler")
    source = kwargs.get("source")
    accountIdFilter = kwargs.get("account_id_filter")
    if handler is not None:
        taskContent["handler"] = handler.asDict()
    if source is not None:
        taskContent["source"] = source.asDict()
    if not withoutAccountFilters:
        taskContent["filters"] = {}
        if accountIdFilter is not None:
            taskContent["filters"] = {"account_id": accountIdFilter}
    return taskContent


def getReporterTaskContentForSchedule(**kwargs) -> dict:
    """
    Get reporter task content as dict
    Args:
        **kwargs: kwargs from user request

    Returns:
        dict with content
    """
    csvDelimiter = kwargs.get("csvDelimiter")
    columns = kwargs.get("columns")
    saveImages = kwargs.get("saveImages")
    taskContent = {"csv_delimiter": csvDelimiter}
    if columns is not None:
        taskContent["columns"] = columns
    if saveImages is not None:
        taskContent["save_images"] = saveImages
    return taskContent


def getTaskPayload(
    taskType: str,
    accountId: Optional[str] = None,
    description: Optional[str] = None,
    withoutAccountFilters: bool = False,
    notificationPolicy: Optional[TaskNotificationPolicy] = None,
    **contentKwargs,
) -> RequestPayload:
    """
    Get task data from kwargs
    Args:
        taskType: task type by name (linker/gc)
        accountId: account id
        description: task description
        withoutAccountFilters: whether account id filter is not allowed (for luna-api)
        notificationPolicy: task notification policy
    Keyword Args:
        task content kwargs
    Returns:
        task data
    """
    if taskType == "linker":
        taskContent = getLinkerTaskContent(**contentKwargs)
    elif taskType == "gc":
        taskContent = getGCTaskContent(**contentKwargs)
    elif taskType == "clustering":
        taskContent = getClusteringTaskContent(**contentKwargs)
    elif taskType == "exporter":
        taskContent = getExporterTaskContent(**contentKwargs)
    elif taskType == "additional_extract":
        taskContent = getAETaskContent(**contentKwargs)
    elif taskType == "estimator":
        taskContent = getEstimatorTaskContent(withoutAccountFilters=withoutAccountFilters, **contentKwargs)
    elif taskType == "cross_match":
        taskContent = getCMTaskContent(**contentKwargs)
    elif taskType == "roc":
        taskContent = getROCTaskContent(**contentKwargs)
    elif taskType == "lambda":
        taskContent = contentKwargs["content"]
    else:
        raise RuntimeError("Unsupported task type")

    data = {"content": taskContent}
    if notificationPolicy is not None:
        data["notification_policy"] = notificationPolicy.asDict()
    if description is not None:
        data["description"] = description

    payload = RequestPayload.buildJson(body=data)
    if accountId is not None:
        payload.applyAccountIdHeader(accountId)

    return payload


def getTaskSchedulePayload(
    taskType: int,
    cron: str,
    cronTz: Optional[str] = None,
    startImmediately: Optional[bool] = None,
    createStopped: Optional[bool] = None,
    accountId: Optional[str] = None,
    withoutAccountFilters: bool = False,
    taskNotificationPolicy: Optional[TaskNotificationPolicy] = None,
    **contentKwargs,
) -> RequestPayload:
    """
    Get task schedule payload
    Args:
        taskType: task type
        accountId: account id
        cron: cron string
        cronTz: cron timezone
        startImmediately: start task immediately
        createStopped: create task stopped schedule task
        withoutAccountFilters: whether account id filter is not allowed (for luna-api)
        taskNotificationPolicy: task notification policy
    Keyword Args:
        contentKwargs: task content kwargs
    Returns:
        prepared task schedule payload
    """
    if taskType == 1:
        taskContent = getLinkerTaskContent(**contentKwargs)
    elif taskType == 2:
        taskContent = getLinkerTaskContent(**contentKwargs)
    elif taskType == 3:
        taskContent = {
            "clusterization": getClusteringTaskContent(**contentKwargs),
            "reporter": getReporterTaskContentForSchedule(**contentKwargs),
        }
    elif taskType == 4:
        taskContent = getGCTaskContent(**contentKwargs)
    elif taskType == 5:
        taskContent = getAETaskContent(**contentKwargs)
    elif taskType == 6:
        taskContent = getCMTaskContent(**contentKwargs)
    elif taskType == 7:
        taskContent = getROCTaskContent(**contentKwargs)
    elif taskType == 8:
        taskContent = getExporterTaskContent(**contentKwargs)
    elif taskType == 9:
        taskContent = getEstimatorTaskContent(withoutAccountFilters, **contentKwargs)
    elif taskType == 10:
        taskContent = contentKwargs["content"]
    else:
        raise RuntimeError("Unsupported task type")

    body = {"task": {"task_type": taskType, "content": taskContent}, "trigger": {"cron": cron}}
    if taskNotificationPolicy is not None:
        body["task"]["notification_policy"] = taskNotificationPolicy.asDict()
    if cronTz is not None:
        body["trigger"]["cron_timezone"] = cronTz
    if startImmediately is not None or createStopped is not None:
        body["behaviour"] = {}
        if startImmediately is not None:
            body["behaviour"]["start_immediately"] = startImmediately
        if createStopped is not None:
            body["behaviour"]["create_stopped"] = createStopped

    payload = RequestPayload.buildJson(body=body)
    if accountId is not None:
        payload.applyAccountIdHeader(accountId)

    return payload
